﻿//////////////////////////////////////////////
// Node.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkExport
#include "../Dll/DllDefines.h"

// Standards
#include <string>
#include <unordered_map>
#include <utility>
#include <vector>

/// Internal classes -------------------------

namespace nkExport
{
	union NodeValueData
	{
		int intValue ;
		unsigned int uintValue ;
		long long longValue ;
		unsigned long long ulongValue ;

		float floatValue ;
		double doubleValue ;

		bool boolValue ;
	} ;

	enum NODE_VALUE_TYPE
	{
		NOT_VALUE_TYPE,
		BOOL_TYPE,
		STRING_TYPE,
		INT_TYPE,
		UINT_TYPE,
		LONG_TYPE,
		ULONG_TYPE,
		FLOAT_TYPE,
		DOUBLE_TYPE
	} ;

	enum NODE_NATURE
	{
		EMPTY_NATURE = 0,
		VALUE_NATURE,
		OBJECT_NATURE,
		ARRAY_NATURE
	} ;
}

/// Class ------------------------------------
	
namespace nkExport
{
	class DLL_EXPORT_EXPORT Node final
	{
		private :
		
			// Attributs
			// On peut avoir des valeurs
			std::pair<NodeValueData, NODE_VALUE_TYPE> _value ;
			// Cas particulier pour own la mémoire
			std::string _valueStr ;
			// Si on est un object
			std::unordered_map<std::string, Node*> _object ;
			// Ou un array
			std::vector<Node*> _array ;

			// Ce qu'on a dans le node actuellement
			NODE_NATURE _nodeNature ;

			// Données user possibles
			void* _userData  ;
			
		public :

			// Constructeur, destructeur
			Node () ;
			~Node () ;

			// Getters
			// Globaux
			NODE_NATURE getNodeNature () const ;
			NODE_VALUE_TYPE getNodeValueType () const ;
			// Utils
			bool isValueTypeNumber () const ;
			// Relatifs a la valeur
			std::string_view getValueAsString () const ;
			int getValueAsInt () const ;
			unsigned int getValueAsUint () const ;
			long long getValueAsLong () const ;
			unsigned long long getValueAsUlong () const ;
			float getValueAsFloat () const ;
			double getValueAsDouble () const ;
			bool getValueAsBool () const ;
			// Object
			unsigned int getMemberCount () const ;
			std::string_view getMemberKey (unsigned int index) ;
			Node* getMemberValue (unsigned int index) ;
			Node* getMemberValue (const std::string_view& key) ;
			// Array
			unsigned int getArraySize () const ;
			Node* getArrayElement (unsigned int index) ;
			// Les données user liées
			void* getUserData () const ;

			// Setters
			// Value
			void setAsString (const std::string_view& value) ;
			void setAsInt (int value) ;
			void setAsUint (unsigned int value) ;
			void setAsLong (long long value) ;
			void setAsUlong (unsigned long long value) ;
			void setAsFloat (float value) ;
			void setAsDouble (double value) ;
			void setAsBool (bool value) ;
			// Object
			Node* addMember (const std::string_view& key) ;
			void deleteMember (const std::string_view& key) ;
			// Array
			Node* addElement () ;
			// User data
			void setUserData (void* value) ;
	} ;
}