﻿//////////////////////////////////////////////
// RenderContext.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Compositor ;
	class Texture ;
	class Viewport ;
}

namespace nkWinUi
{
	class GraphicsWindow ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT RenderContext
	{
		protected :
		
			// Attributs
			// Le Backbuffer
			Texture* _backBuffer ;
			// Depth Target pour confort
			Texture* _depthBuffer ;
			// Le Viewport
			Viewport* _viewport ;

			// Le compositor utilisé si besoin est
			Compositor* _compositor ;

			// La HWND attachée à la base
			nkWinUi::GraphicsWindow* _attachedWin ;
			bool _ownsWindow ;
		
		public :
		
			// Constructeur, destructeur
			RenderContext () ;
			virtual ~RenderContext () ;

			// Getters
			Texture* getBackBuffer () const ;
			Texture* getDepthBuffer () const ;
			Viewport* getViewport () const ;
			Compositor* getCompositor () const ;
			int getContextWidth () const ;
			int getContextHeight () const ;
			bool isWindowFocused () const ;
			nkWinUi::GraphicsWindow* getAttachedWin () const ;
			bool getOwnsWindow () const ;

			// Setters
			void setCompositor (Compositor* value) ;
			void setOwnsWindow (bool value) ;
			virtual void setGammaCorrected (bool value) = 0 ;

			// Quand il faut initialiser...
			virtual void initialize (nkWinUi::GraphicsWindow* window) = 0 ;

			// Pour le rendu
			virtual void swap () = 0 ;

			// Manipulation chain
			virtual void refreshBufferSize () = 0 ;
			virtual void refreshViewportSize () ;

			// TODO si jamais les perfos valent le coup
			//void enableUavManipulation (bool value) ;
	} ;
}