﻿//////////////////////////////////////////////
// ResourceManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkResources
#include "Dll/DllDefines.h"

// Standards
#include <string>
#include <string_view>
#include <vector>

/// Class ------------------------------------

namespace nkResources
{
	class DLL_RESOURCES_EXPORT ResourceManager final : public nkCommon::SingletonClass<ResourceManager>
	{
		friend class nkCommon::SingletonClass<ResourceManager> ;

		private :

			// Attributs
			// Le dossier de travail
			std::string _workingPath ;
			std::string _workingPathAbsolute ;
			// Le dossier de lancement
			std::string _launchedPath ;

		private :

			// Functions
			// Constructeur, destructeur
			ResourceManager() ;
			~ResourceManager () ;

		public :

			// Le working path
			void setWorkingPath (const std::string_view& path) ;
			void setWorkingPathAbsolute (const std::string_view& path) ;
			const char* getWorkingPath () const ;
			const char* getWorkingPathAbsolute () const ;

			// Path de lancement
			const char* getLaunchedPath () const ;
			void setLaunchedPath (const std::string_view& path) ;

			// Traduction chemin absolu en relatif
			std::string getRelativeFromAbsolute (const std::string_view& absolutePath) const ;
			std::string getAbsoluteFromRelative (const std::string_view& relativePath) const ;
			std::string getRelativeFromWorkingDir (const std::string_view& workingPath) const ;
			std::string getAbsoluteFromWorkingDir (const std::string_view& workingPath) const ;
			std::string getFromWorkingDirFromRelative (const std::string_view& relativePath) const ;
			std::string getFromWorkingDirFromAbsolute (const std::string_view& absolutePath) const ;

			// Check de folder
			bool checkFileExists (const std::string_view& filePathAbs) const ;
			void checkFolderExists (const char* path) ;

			// Chargement de ressource
			std::vector<unsigned char> loadFileIntoMemory (const std::string_view& filePathAbs, bool silent = false) ;
	} ;
}